{% from 'macros.jinja2' import base_type %}

<ul class="mkapi-args-list">
{% for item in section.items %}
<li>{% if item.name %}<code class="mkapi-args-name">{{ item.name }}</code>{% endif %}
{{ base_type(item) }}
<span class="mkapi-args-dash">&mdash;</span> <span class="mkapi-args-desc">{{ item.html|safe }}</span></li>
{% endfor %}
</ul>
